"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Channel = void 0;
var request = require('request');
var presence_channel_1 = require("./presence-channel");
var private_channel_1 = require("./private-channel");
var log_1 = require("./../log");
var fs = require('fs');
var Channel = (function () {
    function Channel(io, options) {
        this.io = io;
        this.options = options;
        this._privateChannels = ['private-*', 'presence-*'];
        this._clientEvents = ['client-*'];
        this.private = new private_channel_1.PrivateChannel(options);
        this.presence = new presence_channel_1.PresenceChannel(io, options);
        this.request = request;
        this.options = options;
        if (this.options.devMode) {
            log_1.Log.success('Channels are ready.');
        }
    }
    Channel.prototype.join = function (socket, data) {
        if (data.channel) {
            if (this.isPrivate(data.channel)) {
                this.joinPrivate(socket, data);
            }
            else {
                socket.join(data.channel);
                this.onJoin(socket, data.channel, data.auth);
            }
        }
    };
    Channel.prototype.clientEvent = function (socket, data) {
        try {
            data = JSON.parse(data);
        }
        catch (e) {
            data = data;
        }
        if (data.event && data.channel) {
            if (this.isClientEvent(data.event) &&
                this.isPrivate(data.channel) &&
                this.isInChannel(socket, data.channel)) {
                this.io.sockets.connected[socket.id]
                    .broadcast.to(data.channel)
                    .emit(data.event, data.channel, data.data);
                if (data.event != 'client-typing') {
                    data.data.event = data.event;
                    this.hook(socket, data.channel, data.auth, "client_event", data.data);
                }
            }
        }
    };
    Channel.prototype.leave = function (socket, channel, reason, auth) {
        return __awaiter(this, void 0, void 0, function () {
            var user_1, member, payload_1;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!channel) return [3, 3];
                        user_1 = null;
                        if (!this.isPresence(channel)) return [3, 2];
                        return [4, this.presence.leave(socket, channel)];
                    case 1:
                        member = _a.sent();
                        if (member !== undefined) {
                            user_1 = member;
                        }
                        _a.label = 2;
                    case 2:
                        socket.leave(channel);
                        if (this.options.devMode) {
                            log_1.Log.info("[" + new Date().toISOString() + "] - " + socket.id + " left channel: " + channel + " (" + reason + ")");
                        }
                        if (user_1 !== null) {
                            payload_1 = { "userId": user_1.user_id };
                        }
                        else {
                            payload_1 = {};
                        }
                        if (this.isPresence(channel) && user_1 !== null) {
                            this.presence.getMembers(channel).then(function (members) {
                                var member = members.find(function (member) { return member.user_id === user_1.user_id; });
                                if (!member) {
                                    _this.hook(socket, channel, auth, "leave", payload_1);
                                }
                            });
                        }
                        else {
                            this.hook(socket, channel, auth, "leave", payload_1);
                        }
                        _a.label = 3;
                    case 3: return [2];
                }
            });
        });
    };
    Channel.prototype.isPrivate = function (channel) {
        var isPrivate = false;
        this._privateChannels.forEach(function (privateChannel) {
            var regex = new RegExp(privateChannel.replace('\*', '.*'));
            if (regex.test(channel))
                isPrivate = true;
        });
        return isPrivate;
    };
    Channel.prototype.joinPrivate = function (socket, data) {
        var _this = this;
        this.private.authenticate(socket, data).then(function (res) {
            socket.join(data.channel);
            if (_this.isPresence(data.channel)) {
                var member = res.channel_data;
                try {
                    member = JSON.parse(res.channel_data);
                }
                catch (e) { }
                _this.presence.join(socket, data.channel, member);
            }
            _this.onJoin(socket, data.channel, data.auth, member);
        }, function (error) {
            if (_this.options.devMode) {
                log_1.Log.error(error.reason);
            }
            _this.io.sockets.to(socket.id)
                .emit('subscription_error', data.channel, error.status);
        });
    };
    Channel.prototype.isPresence = function (channel) {
        return channel.lastIndexOf('presence-', 0) === 0;
    };
    Channel.prototype.onJoin = function (socket, channel, auth, member) {
        var _this = this;
        if (member === void 0) { member = null; }
        if (this.options.devMode) {
            log_1.Log.info("[" + new Date().toISOString() + "] - " + socket.id + " joined channel: " + channel);
        }
        var payload;
        if (member !== null) {
            payload = { "userId": member.user_id };
        }
        else {
            payload = {};
        }
        if (this.isPresence(channel) && member !== null) {
            this.presence.getMembers(channel).then(function (members) {
                var user = members.find(function (user) { return user.user_id === member.user_id; });
                if (!user) {
                    _this.hook(socket, channel, auth, "join", payload);
                }
            });
        }
        else {
            this.hook(socket, channel, auth, "join", payload);
        }
    };
    Channel.prototype.isClientEvent = function (event) {
        var isClientEvent = false;
        this._clientEvents.forEach(function (clientEvent) {
            var regex = new RegExp(clientEvent.replace('\*', '.*'));
            if (regex.test(event))
                isClientEvent = true;
        });
        return isClientEvent;
    };
    Channel.prototype.isInChannel = function (socket, channel) {
        return !!socket.rooms[channel];
    };
    Channel.prototype.hook = function (socket, channel, auth, event, payload) {
        var _this = this;
        if (typeof this.options.hookEndpoint == 'undefined' ||
            !this.options.hookEndpoint) {
            return;
        }
        var hookEndpoint = this.options.hookEndpoint;
        var options = this.prepareHookHeaders(socket, auth, channel, hookEndpoint, event, payload);
        this.request.post(options, function (error, response, body, next) {
            if (error) {
                if (_this.options.devMode) {
                    log_1.Log.error("[" + new Date().toISOString() + "] - Error call " + event + " hook " + socket.id + " for " + options.form.channel);
                }
                log_1.Log.error(error);
            }
            else if (response.statusCode !== 200) {
                if (_this.options.devMode) {
                    log_1.Log.warning("[" + new Date().toISOString() + "] - Error call " + event + " hook " + socket.id + " for " + options.form.channel);
                    log_1.Log.error(response.body);
                }
            }
            else {
                if (_this.options.devMode) {
                    log_1.Log.info("[" + new Date().toISOString() + "] - Call " + event + " hook for " + socket.id + " for " + options.form.channel + ": " + response.body);
                }
            }
        });
    };
    Channel.prototype.prepareHookHeaders = function (socket, auth, channel, hookEndpoint, event, payload) {
        var hookHost = this.options.hookHost ? this.options.hookHost : this.options.authHost;
        var options = {
            url: hookHost + hookEndpoint,
            form: {
                event: event,
                channel: channel,
                payload: payload
            },
            headers: (auth && auth.headers) ? auth.headers : {}
        };
        if (hookHost.indexOf('https') > -1 && this.options.sslCertPath && this.options.sslKeyPath) {
            options['agentOptions'] = {
                cert: fs.readFileSync(this.options.sslCertPath),
                key: fs.readFileSync(this.options.sslKeyPath),
                passphrase: this.options.sslPassphrase,
            };
            options['strictSSL'] = false;
        }
        options.headers['Cookie'] = socket.request.headers.cookie;
        options.headers['X-Requested-With'] = 'XMLHttpRequest';
        return options;
    };
    return Channel;
}());
exports.Channel = Channel;
